#!/usr/bin/env python3
#
#===- clang-tidy-diff.py - ClangTidy Diff Checker ------------*- python -*--===#
#
#                     The LLVM Compiler Infrastructure
#
# This file is distributed under the University of Illinois Open Source
# License. See LICENSE.TXT for details.
#
#===------------------------------------------------------------------------===#

r"""
ClangTidy Diff Checker
======================

This script reads input from a unified diff, runs clang-tidy on all changed
files and outputs clang-tidy warnings in changed lines only. This is useful to
detect clang-tidy regressions in the lines touched by a specific patch.
Example usage for git/svn users:

  git diff -U0 HEAD^ | clang-tidy-diff.py -p1
  svn diff --diff-cmd=diff -x-U0 | \
      clang-tidy-diff.py -fix -checks=-*,modernize-use-override

"""

import argparse
import json
import re
import subprocess
import sys


def main():
  parser = argparse.ArgumentParser(description=
                                   'Run clang-tidy against changed files, and '
                                   'output diagnostics only for modified '
                                   'lines.')
  parser.add_argument('-clang-tidy-binary', metavar='PATH',
                      default='clang-tidy',
                      help='path to clang-tidy binary')
  parser.add_argument('-p', metavar='NUM', default=0,
                      help='strip the smallest prefix containing P slashes')
  parser.add_argument('-regex', metavar='PATTERN', default=None,
                      help='custom pattern selecting file paths to check '
                      '(case sensitive, overrides -iregex)')
  parser.add_argument('-iregex', metavar='PATTERN', default=
                      r'.*\.(cpp|cc|c\+\+|cxx|c|cl|h|hpp|m|mm|inc)',
                      help='custom pattern selecting file paths to check '
                      '(case insensitive, overridden by -regex)')

  parser.add_argument('-fix', action='store_true', default=False,
                      help='apply suggested fixes')
  parser.add_argument('-checks',
                      help='checks filter, when not specified, use clang-tidy '
                      'default',
                      default='')
  parser.add_argument('-format-style',
                      help='style format',
                      default='')
  parser.add_argument('-path', dest='build_path',
                      help='Path used to read a compile command database.')
  parser.add_argument('-extra-arg', dest='extra_arg',
                      action='append', default=[],
                      help='Additional argument to append to the compiler '
                      'command line.')
  parser.add_argument('-extra-arg-before', dest='extra_arg_before',
                      action='append', default=[],
                      help='Additional argument to prepend to the compiler '
                      'command line.')
  parser.add_argument('-quiet', action='store_true', default=False,
                      help='Run clang-tidy in quiet mode')
  clang_tidy_args = []
  argv = sys.argv[1:]
  if '--' in argv:
    clang_tidy_args.extend(argv[argv.index('--'):])
    argv = argv[:argv.index('--')]

  args = parser.parse_args(argv)

  # Extract changed lines for each file.
  filename = None
  lines_by_file = {}
  for line in sys.stdin:
    match = re.search('^\+\+\+\ \"?(.*?/){%s}([^ \t\n\"]*)' % args.p, line)
    if match:
      filename = match.group(2)
    if filename == None:
      continue

    if args.regex is not None:
      if not re.match('^%s$' % args.regex, filename):
        continue
    else:
      if not re.match('^%s$' % args.iregex, filename, re.IGNORECASE):
        continue

    match = re.search('^@@.*\+(\d+)(,(\d+))?', line)
    if match:
      start_line = int(match.group(1))
      line_count = 1
      if match.group(3):
        line_count = int(match.group(3))
      if line_count == 0:
        continue
      end_line = start_line + line_count - 1;
      lines_by_file.setdefault(filename, []).append([start_line, end_line])

  if len(lines_by_file) == 0:
    print("No relevant changes found.")
    sys.exit(0)

  line_filter_json = json.dumps(
    [{"name" : name, "lines" : lines_by_file[name]} for name in lines_by_file],
    separators = (',', ':'))

  quote = "";
  if sys.platform == 'win32':
    line_filter_json=re.sub(r'"', r'"""', line_filter_json)
  else:
    quote = "'";

  # Run clang-tidy on files containing changes.
  command = [args.clang_tidy_binary]
  command.append('-line-filter=' + quote + line_filter_json + quote)
  if args.fix:
    command.append('-fix')
  if args.checks != '':
    command.append('-checks=' + quote + args.checks + quote)
  if args.format_style != '':
    command.append('-format-style=' + quote + args.format_style + quote)
  if args.quiet:
    command.append('-quiet')
  if args.build_path is not None:
    command.append('-p=%s' % args.build_path)
  command.extend(lines_by_file.keys())
  for arg in args.extra_arg:
      command.append('-extra-arg=%s' % arg)
  for arg in args.extra_arg_before:
      command.append('-extra-arg-before=%s' % arg)
  command.extend(clang_tidy_args)

  sys.exit(subprocess.call(' '.join(command), shell=True))

if __name__ == '__main__':
  main()



CCFLAGS = ['-I/usr/include/lua5.1', '-O2', '-ansi']
#LIBPATH = ['/usr/local/lib']
LIBS = ['lua5.1', 'dl', 'm']
prefix = '/mingw'
#build_dev=1
tolua_bin = 'tolua++5.1'
tolua_lib = 'tolua++5.1'
TOLUAPP = 'tolua++5.1'





## This is the linux configuration file
# use 'scons -h' to see the list of command line options available

# Compiler flags (based on Debian's installation of lua)
#LINKFLAGS = ['-g']
CCFLAGS = ['-O2', '-ansi', '-Wall']
#CCFLAGS = ['-I/usr/include/lua50', '-g']

# this is the default directory for installation. Files will be installed on
# <prefix>/bin, <prefix>/lib and <prefix>/include when you run 'scons install'
#
# You can also specify this directory on the command line with the 'prefix'
# option
#
# You can see more 'generic' options for POSIX systems on config_posix.py

prefix = '/usr/local'

# libraries (based on Debian's installation of lua)
LIBS = ['lua', 'dl', 'm']




#!/usr/bin/env python3

import sys
import os
import time
import yaml
import datetime

base_path = os.path.dirname(os.path.realpath(__file__))

with open(os.path.join(base_path, "config_settings.yaml")) as file:
    config_settings = yaml.safe_load(file)

with open(os.path.join(base_path, "variables.yaml")) as file:
    variables = yaml.safe_load(file)

with open(os.path.join(base_path, "lua.yaml")) as file:
    lua = yaml.safe_load(file)

build_date = datetime.datetime.fromtimestamp(
    int(os.environ.get('SOURCE_DATE_EPOCH', time.time())),
    tz=datetime.timezone.utc,
)
data = {
    "config_settings": config_settings,
    "variables": variables,
    "lua": lua,
    "date": build_date.date().isoformat(),
    "copyright_year": build_date.year,
}

from jinja2 import Environment, FileSystemLoader, select_autoescape


def reverse_format(param_list, format_string):
    return format_string.format(param_list)


env = Environment(
    loader=FileSystemLoader("."),
    autoescape=select_autoescape(),
)
env.filters["reverse_format"] = reverse_format

template = env.get_template(sys.argv[1])
print(template.render(data))



#!/usr/bin/env python

import sys

if len(sys.argv) <= 1:
  print >> sys.stderr, "Usage: ./changelog2html.py [changelog file]"
  sys.exit(1)

f = sys.argv[1]

blah = 0

print '<html><head><meta http-equiv="content-type" content="text/html;charset=utf-8" /></head><body>'

for i in open(f).read().splitlines():
  # ignore empty lines

  if i and i[0].isspace():
    if not '*' in i:
      print '      ' + i.strip()
    else:
      s = i.split('*', 1)[1].strip()
      print '  <li>' + s.replace('<', '&lt;').replace('>', '&gt;')
  else:
    if blah:
      print '</ul>'
    print '<h3>%s</h3>' % i.strip()
    print '<ul>'
    blah = 1

if blah:
  print '</ul>'

print '</html></body>'



#!/usr/bin/env python3

import argparse
import os
import sys
import subprocess


def check_file(filename, excludes, extensions):
    """
    Check if a file should be included in our check
    """
    name, ext = os.path.splitext(filename)

    if len(ext) > 0 and ext in extensions:
        if len(excludes) == 0:
            return True

        for exclude in excludes:
            if exclude in filename:
                return False

        return True

    return False


def check_directory(directory, excludes, extensions):
    output = []

    if len(excludes) > 0:
        for exclude in excludes:
            if exclude in directory:
                directory_excluded = False
                return output

    for root, _, files in os.walk(directory):
        for file in files:
            filename = os.path.join(root, file)
            if check_file(filename, excludes, extensions):
                print("Will check file [{}]".format(filename))
                output.append(filename)
    return output

def get_git_root(git_bin):
    cmd = [git_bin, "rev-parse", "--show-toplevel"]
    try:
        return subprocess.check_output(cmd).strip()
    except subprocess.CalledProcessError, e:
        print("Error calling git [{}]".format(e))
        raise

def clean_git_filename(line):
    """
    Takes a line from git status --porcelain and returns the filename
    """
    file = None
    git_status = line[:2]
    # Not an exhaustive list of git status output but should
    # be enough for this case
    # check if this is a delete
    if 'D' in git_status:
        return None
    # ignored file
    if '!' in git_status:
        return None
    # Covers renamed files
    if '->' in line:
        file = line[3:].split('->')[-1].strip()
    else:
        file = line[3:].strip()

    return file


def get_changed_files(git_bin, excludes, file_extensions):
    """
    Run git status and return the list of changed files
    """
    extensions = file_extensions.split(",")
    # arguments coming from cmake will be *.xx. We want to remove the *
    for i, extension in enumerate(extensions):
        if extension[0] == '*':
            extensions[i] = extension[1:]

    git_root = get_git_root(git_bin)

    cmd = [git_bin, "status", "--porcelain", "--ignore-submodules"]
    print("git cmd = {}".format(cmd))
    output = []
    returncode = 0
    try:
        cmd_output = subprocess.check_output(cmd)
        for line in cmd_output.split('\n'):
            if len(line) > 0:
                file = clean_git_filename(line)
                if not file:
                    continue
                file = os.path.join(git_root, file)

                if file[-1] == "/":
                    directory_files = check_directory(
                        file, excludes, file_extensions)
                    output = output + directory_files
                else:
                    if check_file(file, excludes, file_extensions):
                        print("Will check file [{}]".format(file))
                        output.append(file)

    except subprocess.CalledProcessError, e:
        print("Error calling git [{}]".format(e))
        returncode = e.returncode

    return output, returncode


def run_clang_format(clang_format_bin, changed_files):
    """
    Run clang format on a list of files
    @return 0 if formatted correctly.
    """
    if len(changed_files) == 0:
        return 0
    cmd = [clang_format_bin, "-style=file",
           "-output-replacements-xml"] + changed_files
    print("clang-format cmd = {}".format(cmd))
    try:
        cmd_output = subprocess.check_output(cmd)
        if "replacement offset" in cmd_output:
            print("ERROR: Changed files don't match format")
            return 1
    except subprocess.CalledProcessError, e:
        print("Error calling clang-format [{}]".format(e))
        return e.returncode

    return 0


def cli():
    # global params
    parser = argparse.ArgumentParser(prog='clang-format-check-changed',
                                     description='Checks if files changed in git match the .clang-format specification')
    parser.add_argument("--file-extensions", type=str,
                        default=".cpp,.h,.cxx,.hxx,.hpp,.cc,.ipp",
                        help="Comma separated list of file extensions to check")
    parser.add_argument('--exclude', action='append', default=[],
                        help='Will not match the files / directories with these in the name')
    parser.add_argument('--clang-format-bin', type=str, default="clang-format",
                        help="The clang format binary")
    parser.add_argument('--git-bin', type=str, default="git",
                        help="The git binary")
    args = parser.parse_args()

    # Run gcovr to get the .gcda files form .gcno
    changed_files, returncode = get_changed_files(
        args.git_bin, args.exclude, args.file_extensions)
    if returncode != 0:
        return returncode

    return run_clang_format(args.clang_format_bin, changed_files)

if __name__ == '__main__':
    sys.exit(cli())



#!/usr/bin/env python3

import subprocess

def get_first_number(s):
    number = ""
    found_digit = False

    for char in s:
        if char.isdigit():
            number += char
            found_digit = True
        elif found_digit:
            break

    return int(number) if number else None


def git_log():
    entries = []
    try:
        result = subprocess.run(
            ["git", "log", "--no-merges", "--shortstat", "--pretty=format:%H %n %s"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            check=True,
        )
        entries = result.stdout.split("\n\n")
    except subprocess.CalledProcessError as e:
        print(f"Error running git command: {e.stderr}")
        return []

    commits = []
    for line in entries:
        info = line.split("\n")
        changes = info[2].split(",")
        files_changed = get_first_number(changes[0])
        insertions = 0
        deletions = 0
        if len(changes) > 1:
            if "insertions" in changes[1]:
                insertions = get_first_number(changes[1])
            elif "deletions" in changes[1]:
                deletions = get_first_number(changes[1])
        if len(changes) > 2:
            if "insertions" in changes[2]:
                insertions = get_first_number(changes[2])
            elif "deletions" in changes[2]:
                deletions = get_first_number(changes[2])
        commit = {
            "hash": info[0].strip(),
            "message": info[1].strip(),
            "changes": {
                "files": files_changed,
                "insertions": insertions,
                "deletions": deletions,
            },
        }
        commits.append(commit)

    return commits

def main():
    log = git_log()
    if len(log) == 0:
        return

    large_changes = sorted(log, key=lambda it: it["changes"]["insertions"] + it["changes"]["deletions"], reverse=True)

    print(f"{'Hash':41} {'Insertions':11} {'Deletions':11} {'Message'}")
    print("-" * 90)
    for commit in large_changes:
        print(
            f"{commit['hash']:41} {commit["changes"]['insertions']:11} {commit["changes"]['deletions']:11} {commit['message']}"
        )


if __name__ == "__main__":
    main()




## This is the POSIX configuration file (will be included for cygwin, mingw
# and possibly mac OSX and BSDs)
# use 'scons -h' to see the list of command line options available

# flags for the compiler
#CCFLAGS = []
CCFLAGS = ['-O2', '-ansi', '-Wall']

# this is the default directory for installation. Files will be installed on
# <prefix>/bin, <prefix>/lib and <prefix>/include when you run 'scons install'
#
# You can also specify this directory on the command line with the 'prefix'
# option

prefix = '/usr/local'

# libraries
LIBS = ['lua', 'lualib', 'm']



import os




## This is the MSVC configuration file
# use 'scons -h' to see the list of command line options available

# flags for the compiler
CCFLAGS = ['/nologo']

# this is the default directory for installation. Files will be installed on
# <prefix>/bin, <prefix>/lib and <prefix>/include when you run 'scons install'
#
# You can also specify this directory on the command line with the 'prefix'
# option
#
# If you leave it as 'None', we'll try to auto-detect it (as 'PLATFORMSDKDIR'
# detected by SCons).

prefix = None # (it's a string)

# the libraries
LIBS = ['lua', 'lualib']

# linkflags
LINKFLAGS = ['/nologo']

## We need to specifiy the environment for the PATH and LIB and all those
# parameters cl tales from it
import os
ENV = os.environ
